#!/usr/bin/env python

"""A renderer for Euclidean graphs"""

#
#  Copyright Thomas Dickerson, 2012.
#  This file is part of PyZPR.
#
#  PyZPR is free software: you can redistribute it and/or modify it
#  under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  PyZPR is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  long with PyZPR.  If not, see <http://www.gnu.org/licenses/>.
#

import sys,math

try:
	import numpy
	from numpy import array, float32, matrix, linalg, empty, uint32
except:
	print "Oops, please install numpy on this machine!"
	sys.exit(1)


try:
	from OpenGL.GLUT import GLUT_DEPTH, GLUT_DOUBLE, GLUT_RGBA
	from OpenGL.GLUT import glutInit, glutInitDisplayMode, glutInitWindowPosition, glutInitWindowSize, glutCreateWindow
	from OpenGL.GLUT import glutDisplayFunc, glutIdleFunc, glutKeyboardFunc, glutSpecialFunc, glutMainLoop, glutSwapBuffers
		
	from OpenGL.GL import GL_COLOR_BUFFER_BIT, GL_DEPTH_BUFFER_BIT, GL_VERTEX_ARRAY, GL_LINE_SMOOTH, GL_CULL_FACE, GL_BLEND, GL_SRC_ALPHA, GL_ONE, GL_ONE_MINUS_SRC_ALPHA
	from OpenGL.GL import GL_POINTS, GL_LINES, GL_LINE_STRIP, GL_LINE_LOOP, GL_TRIANGLES, GL_TRIANGLE_STRIP, GL_TRIANGLE_FAN, GL_QUADS, GL_QUAD_STRIP, GL_POLYGON
	from OpenGL.GL import glClear, glEnableClientState, glDisableClientState, glVertexPointerf, glDrawArrays
	from OpenGL.GL import glEnable, glDisable, glLineWidth, glColor4f, glBlendFunc
	import zpr
except:
	print "Oops, please install PyOpenGL on this machine!"
	sys.exit(1)
	
BLUE = (0., 0., 1., 1.)
PURPLE = (1., 0., 1., 0.5)  # Last number is transparency, more transparent the lower the number;  1 = opaque and 0 = invisible
LIGHT_BLUE = (0., 1., 1., 1.,)

BOX_WIDTH = 4.
MODEL_WIDTH = 3.

def alpha_sort(thing1, thing2):
	(d1, l1, (r1,g1,b1,a1)), v1 = thing1
	(d2, l2, (r2,g2,b2,a2)), v2 = thing2
	return int(math.copysign(1,a1 - a2))

	
def renderScene():
	global data
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

	glEnable(GL_LINE_SMOOTH)
	glEnable(GL_BLEND)
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
	glDisable(GL_CULL_FACE)
	glEnableClientState(GL_VERTEX_ARRAY)
	
	for (data_type, line_width, color), verts in sorted(data.items(), cmp=alpha_sort, reverse=True):
		glLineWidth(line_width)
		glColor4f(*color)
		glVertexPointerf(verts)
		glDrawArrays(data_type, 0, verts.shape[0])

	glDisableClientState(GL_VERTEX_ARRAY)
	glEnable(GL_CULL_FACE)
	glDisable(GL_BLEND)
	glDisable(GL_LINE_SMOOTH)
	
	glutSwapBuffers()
	
	
def handleKeyEvent(char, mouseX, mouseY):
	if char.upper() == 'Q':
		sys.exit(0)

def handleSpecialKey(code, mouseX, mouseY):
	pass

def main(*argv):
	"""Takes an arbitrary number of arguments with the following structure ((line_width, (r, g, b)), vertex_array)"""
	
	global data
	data = {}
	
	glutInit(0);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(30,20);
	glutInitWindowSize(540,360);
	glutCreateWindow("Crystal TurtleBug");

	zpr.init()
	
	glutDisplayFunc(renderScene)
	#glutIdleFunc(renderScene)
	glutKeyboardFunc(handleKeyEvent)
	glutSpecialFunc(handleSpecialKey)
	for style, verts in argv:
		new_mat = matrix(verts.reshape(len(verts)/3,3))
		if style in data:
			data[style] = numpy.row_stack([data[style], new_mat])
		else:
			data[style] = new_mat
	
	glutMainLoop()

if __name__=="__main__":
	main(*sys.argv[1:])
